#
# Copyright (c) 2006, Greggman (http://games.greggman.com)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
# Redistributions of source code must retain the above copyright notice,this
# list of conditions and the following disclaimer.
#
# Redistributions binary form must reproduce the above copyright notice,
# this list ofconditions and the following disclaimer in the documentation
# and/or othermaterials provided with the distribution.
#
# Neither the name of Greggman nor the names of its contributors may be
# used to endorse orpromote products derived from this software without
# specific prior writtenpermission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
# NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# the latest version of this script can hopefully be found at
# http://games.greggman.com/pages/cutsprites/cutsprites.htm
#

use warnings;
use strict;
use IO::Handle;
use Data::Dumper;

sub convertToMayaAscii
{
   my ($inFilename, $outFilename) = @_;

   my $OUTFILE = IO::Handle->new();
   open ($OUTFILE, ">" . $outFilename) || die ("couldn't open ($outFilename)\n");

   my $INFILE = IO::Handle->new();
   open ($INFILE, $inFilename) || die ("couldn't open ($inFilename)\n");

   print $OUTFILE
   '//Maya ASCII 7.0 scene' . "\n" .
   'requires maya "7.0";' . "\n" .
   'currentUnit -l centimeter -a degree -t ntsc;' . "\n" .
   'fileInfo "application" "maya";' . "\n" .
   'fileInfo "product" "Maya Complete 7.0";' . "\n" .
   'fileInfo "version" "7.0";' . "\n" .
   '' . "\n" .
   '';


   my $docWidth  = 0;
   my $docHeight = 0;

   while (<$INFILE>)
   {
      if (/_WIDTH\s+(\d+)/)
      {
         $docWidth = $1;
      }
      if (/_HEIGHT\s+   (\d+)/)
      {
         $docHeight = $1;
      }
      if (/SPRITEINFO\((\d+),(\w+),"([^"]+)",(\d+),(\d+),(\d+),(\d+),(\d+),(\d+),(\d+),(\d+)\)/)
      {
         my $info = {
            'id'           => $1,
            'name'         => $2,
            'filename'     => $3,
            'centerLeft'   => $4,
            'centerTop'    => $5,
            'centerRight'  => $6,
            'centerBottom' => $7,
            'boundsLeft'   => $8,
            'boundsTop'    => $9,
            'boundsRight'  => $10,
            'boundsBottom' => $11,
            };

         my $centerX = ($info->{'centerRight'}  + $info->{'centerLeft'}) / 2;
         my $centerY = ($info->{'centerBottom'} + $info->{'centerTop'})  / 2;

         $info->{'left'}   = $info->{'boundsLeft'}   - $centerX;
         $info->{'right'}  = $info->{'boundsRight'}  - $centerX;
         $info->{'top'}    = $docHeight - ($info->{'boundsTop'}    - $centerY);
         $info->{'bottom'} = $docHeight - ($info->{'boundsBottom'} - $centerY);

         $info->{'x'} = $centerX;
         $info->{'y'} = $docHeight - $centerY;
         $info->{'z'} = 0 - $1;
         $info->{'meshZ'} = 0;

         $info->{'filename'} = substr($info->{'filename'}, 0, length($info->{'filename'}) - 3) . "tga";

         print $OUTFILE makeMayaPoly($info);
      }
   }

   close ($INFILE);
   close ($OUTFILE);
}

sub makeMayaPoly
{
   my ($info) = @_;

   return
'createNode transform -n "billboard_' . $info->{'name'} . '";' . "\n" .
'   setAttr ".t" -type "double3" ' . $info->{'x'} . ' ' . $info->{'y'} . ' ' . $info->{'z'} . ' ;' . "\n" .
'createNode mesh -n "' . $info->{'name'} . 'Shape" -p "billboard_' . $info->{'name'} . '";' . "\n" .
'       setAttr -k off ".v";' . "\n" .
'       setAttr ".vir" yes;' . "\n" .
'       setAttr ".vif" yes;' . "\n" .
'       setAttr ".uvst[0].uvsn" -type "string" "map1";' . "\n" .
'       setAttr -s 4 ".uvst[0].uvsp[0:3]" -type "float2" 0 0 1 0 0 1 1 1;' . "\n" .
'       setAttr ".cuvs" -type "string" "map1";' . "\n" .
'       setAttr ".dcc" -type "string" "Ambient+Diffuse";' . "\n" .
'       setAttr ".covm[0]"  0 1 1;' . "\n" .
'       setAttr ".cdvm[0]"  0 1 1;' . "\n" .
'       setAttr -s 4 ".vt[0:3]"  ' . $info->{'left'} . ' ' . $info->{'bottom'} . ' ' . $info->{'meshZ'} . ' ' . $info->{'right'} . ' ' . $info->{'bottom'} . ' ' . $info->{'meshZ'} . ' ' . $info->{'left'} . ' ' . $info->{'top'} . ' ' . $info->{'meshZ'} . ' ' . $info->{'right'} . ' ' . $info->{'top'} . ' ' . $info->{'meshZ'} . ';' . "\n" .
'       setAttr -s 4 ".ed[0:3]"  0 1 0 0 2 0 1 3 0 2 3 0;' . "\n" .
'       setAttr ".fc[0]" -type "polyFaces" ' . "\n" .
'               f 4 0 2 -4 -2 ' . "\n" .
'               mu 0 4 0 1 3 2 ;' . "\n" .
'       setAttr ".cd" -type "dataPolyComponent" Index_Data Edge 0 ;' . "\n" .
'createNode place2dTexture -n "' . $info->{'name'} . 'place2dTexture";' . "\n" .
'       setAttr ".wu" no;' . "\n" .
'       setAttr ".wv" no;' . "\n" .
'createNode lambert -n "' . $info->{'name'} . 'lambert";' . "\n" .
'createNode shadingEngine -n "' . $info->{'name'} . 'lambertSG";' . "\n" .
'       setAttr ".ihi" 0;' . "\n" .
'       setAttr ".ro" yes;' . "\n" .
'createNode materialInfo -n "' . $info->{'name'} . 'materialInfo";' . "\n" .
'createNode file -n "' . $info->{'name'} . 'file";' . "\n" .
'       setAttr ".ftn" -type "string" "' . $info->{'filename'} . '";' . "\n" .
'//connectAttr "' . $info->{'name'} . 'lambertSG.msg" "lightLinker1.lnk[2].olnk";' . "\n" .
'connectAttr "' . $info->{'name'} . 'file.oc" "' . $info->{'name'} . 'lambert.c";' . "\n" .
'connectAttr "' . $info->{'name'} . 'file.ot" "' . $info->{'name'} . 'lambert.it";' . "\n" .
'connectAttr "' . $info->{'name'} . 'lambert.oc" "' . $info->{'name'} . 'lambertSG.ss";' . "\n" .
'connectAttr "' . $info->{'name'} . 'Shape.iog" "' . $info->{'name'} . 'lambertSG.dsm" -na;' . "\n" .
'connectAttr "' . $info->{'name'} . 'lambertSG.msg" "' . $info->{'name'} . 'materialInfo.sg";' . "\n" .
'connectAttr "' . $info->{'name'} . 'lambert.msg" "' . $info->{'name'} . 'materialInfo.m";' . "\n" .
'connectAttr "' . $info->{'name'} . 'file.msg" "' . $info->{'name'} . 'materialInfo.t" -na;' . "\n" .
'connectAttr "' . $info->{'name'} . 'lambertSG.pa" ":renderPartition.st" -na;' . "\n" .
'connectAttr "' . $info->{'name'} . 'lambert.msg" ":defaultShaderList1.s" -na;' . "\n" .
'connectAttr "' . $info->{'name'} . 'file.msg" ":defaultTextureList1.tx" -na;' . "\n" .
'connectAttr "' . $info->{'name'} . 'place2dTexture.o" "' . $info->{'name'} . 'file.uv";' . "\n" .
'connectAttr "' . $info->{'name'} . 'place2dTexture.ofs" "' . $info->{'name'} . 'file.fs";' . "\n" .
'connectAttr "' . $info->{'name'} . 'place2dTexture.c" "' . $info->{'name'} . 'file.c";' . "\n" .
'connectAttr "' . $info->{'name'} . 'place2dTexture.tf" "' . $info->{'name'} . 'file.tf";' . "\n" .
'connectAttr "' . $info->{'name'} . 'place2dTexture.rf" "' . $info->{'name'} . 'file.rf";' . "\n" .
'connectAttr "' . $info->{'name'} . 'place2dTexture.mu" "' . $info->{'name'} . 'file.mu";' . "\n" .
'connectAttr "' . $info->{'name'} . 'place2dTexture.mv" "' . $info->{'name'} . 'file.mv";' . "\n" .
'connectAttr "' . $info->{'name'} . 'place2dTexture.s" "' . $info->{'name'} . 'file.s";' . "\n" .
'connectAttr "' . $info->{'name'} . 'place2dTexture.wu" "' . $info->{'name'} . 'file.wu";' . "\n" .
'connectAttr "' . $info->{'name'} . 'place2dTexture.wv" "' . $info->{'name'} . 'file.wv";' . "\n" .
'connectAttr "' . $info->{'name'} . 'place2dTexture.re" "' . $info->{'name'} . 'file.re";' . "\n" .
'connectAttr "' . $info->{'name'} . 'place2dTexture.vt1" "' . $info->{'name'} . 'file.vt1";' . "\n" .
'connectAttr "' . $info->{'name'} . 'place2dTexture.vt2" "' . $info->{'name'} . 'file.vt2";' . "\n" .
'connectAttr "' . $info->{'name'} . 'place2dTexture.vt3" "' . $info->{'name'} . 'file.vt3";' . "\n" .
'connectAttr "' . $info->{'name'} . 'place2dTexture.vc1" "' . $info->{'name'} . 'file.vc1";' . "\n" .
'connectAttr "' . $info->{'name'} . 'place2dTexture.n" "' . $info->{'name'} . 'file.n";' . "\n" .
'connectAttr "' . $info->{'name'} . 'place2dTexture.of" "' . $info->{'name'} . 'file.of";' . "\n" .
'connectAttr "' . $info->{'name'} . 'place2dTexture.r" "' . $info->{'name'} . 'file.ro";' . "\n" .
"\n";
}

1;
